/*
 * HARK/hark-base
 *
 * Copyright (C) 2008-2018
 *
 */

#if !defined(HARK_BASE_STREAM_H_INCLUDED)
#define HARK_BASE_STREAM_H_INCLUDED 1

#include "common.h"

#include "Object.h"
#include <iostream>
#include <string>
#include <fstream>
#include <stdio.h>
#include "ObjectParser.h"

namespace HARK
{
    class Stream : public Object
    {
    protected:
	int owner;
	
    public:
	Stream(bool _owner=true)
	    : owner(_owner)
	{}
	virtual ~Stream() {}
    };

    class IStream : virtual public Stream
    {
    private:
	std::istream *int_istream;
	
    public:
	IStream(std::istream *_str, bool _owner=true) :
	    Stream(_owner),
	    int_istream(_str)
	{}

	IStream &read (char *ch, int len)
	{
	    int_istream->read(ch,len);
	    return *this;
	}

	int gcount()
	{
	    return int_istream->gcount();
	}

	int eof()
	{
	    return int_istream->eof();
	}

	int fail()
	{
	    return int_istream->fail();
	}

	IStream &getline (char *ch, int len)
	{
	    int_istream->getline(ch,len);
	    return *this;
	}

	operator std::istream &()
	{
	    return *int_istream;
	}

	void printOn(std::ostream &out) const
	{
	    out << "<IStream unknown>";
	}

	~IStream()
	{
	    if (owner) {
		delete dynamic_cast<std::istream *>(int_istream);
		owner=false;
	    }
	}

	IStream &seekg (int pos, std::ios::seekdir dir)
	{
	    int_istream->seekg(pos, dir);
	    return *this;
	}

	template <class T>
	IStream &operator >> (T &obj)
	{
	    *int_istream >> obj;
	    return *this;
	}
    };

    class OStream : virtual public Stream
    {
    private:
	std::ostream *int_ostream;
	
    public:
	OStream(std::ostream *_str, bool _owner=true) :
	    Stream(_owner),
	    int_ostream(_str)
	{}
	OStream &write (const char *ch, int len)
	{
	    int_ostream->write(ch,len);
	    return *this;
	}

	int eof()
	{
	    return int_ostream->eof();
	}

	int fail()
	{
	    return int_ostream->fail();
	}

	void flush()
	{
	    int_ostream->flush();
	}

	operator std::ostream &()
	{
	    return *int_ostream;
	}

	void printOn(std::ostream &out) const
	{
	    out << "<OStream unknown>";
	}

	~OStream()
	{
	    if (owner) {
		delete dynamic_cast<std::ostream *>(int_ostream);
		owner=false;
	    }
	}

	OStream &seekp (int pos, std::ios::seekdir dir)
	{
	    int_ostream->seekp(pos, dir);
	    return *this;
	}

	template <class T>
	OStream &operator << (const T &obj)
	{
	    *int_ostream << obj;
	    return *this;
	}
    };

    class IOStream : public IStream, public OStream
    {
    private:
	std::iostream *int_iostream;

    public:
	IOStream(std::iostream *_str, bool _owner=true) :
	    Stream(_owner),
	    IStream(_str, _owner),
	    OStream(_str, _owner),
	    int_iostream(_str)
	{}
	int eof()
	{
	    return int_iostream->eof();
	}
	int fail()
	{
	    return int_iostream->fail();
	}
	operator std::iostream &()
	{
	    return *int_iostream;
	}
	void printOn(std::ostream &out) const
	{
	    out << "<IOStream unknown>";
	}
	~IOStream()
	{
	    if (owner) {
		delete dynamic_cast<std::iostream *>(int_iostream);
		owner=false;
	    }
	}

	IOStream &seekg (int pos, std::ios::seekdir dir)
	{
	    int_iostream->seekg(pos, dir);
	    return *this;
	}

	IOStream &seekp (int pos, std::ios::seekdir dir)
	{
	    int_iostream->seekp(pos, dir);
	    return *this;
	}
    };
}

#endif

/* end of file */
