/*
 * HARK/hark-base
 *
 * Copyright (C) 2008-2018
 *
 */

#include "BufferedNode.h"

namespace HARK
{
    OutputCacheInfo::OutputCacheInfo() :
	lookAhead(0),
	lookBack(0)
    {
    }

    InputCacheInfo::InputCacheInfo() :
	lookAhead(0),
	lookBack(0)
    {
    }

    BufferedNode::BufferedNode()
    {
	throw new GeneralException("BufferedNode default constructor should not be called",__FILE__,__LINE__);
    }

    BufferedNode::BufferedNode(const std::string &nodeName, const ParameterSet &params) :
	Node(nodeName, params),
	inOrder(false)
    {
    }

    BufferedNode::~BufferedNode()
    {
    }

    int
    BufferedNode::addOutput(const std::string &outputName)
    {
	unsigned int outID = this->Node::addOutput(outputName);
	if (outID >= outputs.size()) {
	    outputs.resize(outID+1);
	}
	return outID;
    }

    int
    BufferedNode::addInput(const std::string &inputName)
    {
	unsigned int inID = this->Node::addInput(inputName);
	if (inID >= inputsCache.size()) {
	    inputsCache.resize(inID+1);
	}
	return inID;
    }

    void
    BufferedNode::initializeBuffers()
    {
	for (unsigned int i=0; i<outputs.size(); i++) {
	    outputs[i].buffer = RCPtr<Buffer>(new Buffer (outputs[i].lookAhead+outputs[i].lookBack+1));
	}
    }

    void
    BufferedNode::initialize()
    {
	processCount = -1;
	this->Node::initialize();
	this->initializeBuffers();
    }

    ObjectRef
    BufferedNode::getOutput(int output_id, int count)
    {
	try {
	    Buffer &outBuffer = *(outputs[output_id].buffer);
	    if (inOrder) {
		for (int i= outBuffer.getCurrentPos() + 1; i<=count; i++) {
		    calculate (output_id, i, outBuffer);
		}
		if (count > processCount) {
		    processCount = count;
		}
		return outBuffer.get(count);

	    } else {
		if (!outBuffer.isValid(count)) {

		    calculate (output_id, count, outBuffer);
		}
		return outBuffer.get(count);
	    }

	} catch (BaseException *e) {
	    throw e->add(new NodeException (this, "Exception caught in BufferedNode::getOutput", __FILE__, __LINE__));
	}
    }

}

/* end of file */
