/*
 * HARK/hark-base
 *
 * Copyright (C) 2008-2018
 *
 */

#include "Collector.h"

namespace HARK {

    DECLARE_NODE(Collector);
    /*Node

     * @name Collector
     * @category Flow
     * @description Pass through with unlimited number of input/output pairs

     * @input_name INPUT
     * @input_description The input

     * @output_name OUTPUT
     * @output_description The output = The input (same name)

     END*/

    Collector::Collector()
    {
	throw new GeneralException("Collector default constructor should not be called",__FILE__,__LINE__);
    }

    Collector::Collector(const std::string &nodeName, const ParameterSet &params) :
	Node(nodeName, params)
    {
    }

    ObjectRef
    Collector::getOutputNamed(const std::string &outputName, int count)
    {
	for (unsigned int i = 0; i < this->inputs.size(); i++) {
	    if(this->inputs[i].name == outputName) {
		return this->getOutput(i, count);
	    }
	}   
 
	throw new NodeException(this, std::string("Unknown output name :") + outputName, __FILE__, __LINE__);
    }

    void
    Collector::initialize()
    {
	this->Node::initialize();
    }

    int
    Collector::translateInput(const std::string &inputName)
    {
	for (unsigned int i = 0; i < this->inputs.size(); i++) {
	    if(this->inputs[i].name == inputName) {
		return i;
	    }
	}    

	return this->addInput(inputName);
    }

    int
    Collector::translateOutput(const std::string &outputName)
    {
	return this->translateInput(outputName);
    }

    bool
    Collector::hasOutput(int output_id) const
    {
	return(int(inputs.size()) > output_id);
    }

    ObjectRef
    Collector::getOutput(int output_id, int count)
    {
	if(!this->hasOutput(output_id)){
	    throw new NodeException(this,"Unknown output_id",__FILE__,__LINE__);
	}

	try{	
	    int outputID = this->inputs[output_id].outputID;
	    return (this->inputs[output_id].node)->getOutput(outputID,count);

	} catch (BaseException *e){
	    throw e->add(new NodeException (this, "Exception caught in Collector::getOutput", __FILE__, __LINE__));
	}
    }

    void
    Collector::request(int outputID, const ParameterSet &req)
    {
	this->inputs[outputID].node->request(this->inputs[outputID].outputID, req);
    }

    void
    Collector::requestAll(const ParameterSet &req)
    {
	for(int i = 0; i < this->inputs.size(); i++) {
	    this->inputs[i].node->request(this->inputs[i].outputID, req);
	}
    }
}

/* end of file */




