/*
 * HARK/hark-base
 *
 * Copyright (C) 2008-2018
 *
 */

#include "Delay.h"

namespace HARK
{
    DECLARE_NODE(Delay);
    /*Node
     *
     * @name Delay
     * @category Flow
     * @description Delay the input of DELAY iterations. Therefore, it can only be used in iterators.
     *
     * @input_name INPUT
     * @input_description The input object
     *
     * @output_name OUTPUT
     * @output_description The output object = input object with a delay
     *
     * @parameter_name DELAY
     * @parameter_description The delay
     * @parameter_type int
     *
     END*/

    Delay::Delay(const std::string &nodeName, const ParameterSet &params) :
	BufferedNode(nodeName, params)
    {
        this->_inputID  = this->addInput("INPUT");
        this->_outputID = this->addOutput("OUTPUT");
        this->_delay    = dereference_cast<int>(parameters.get("DELAY"));

	if(this->_delay > 0){
	    this->inputsCache[this->_inputID].lookAhead = this->_delay;
	}else{
	    this->inputsCache[this->_inputID].lookBack = -this->_delay;
	}
    }

//    ObjectRef
//    Delay::getOutput(int output_id, int count)
//    {
//	NodeInput input = this->inputs[this->_inputID];
//	if (count - this->_delay >= 0) {
//	    return input.node->getOutput(input.outputID, count - this->_delay);
//	} else {
//	    return nilObject;
//	}
//    }

    void
    Delay::calculate(int output_id, int count, Buffer &buf)
    {
	NodeInput input = this->inputs[this->_inputID];
	if (count - this->_delay >= 0) {
	    buf[count] = input.node->getOutput(input.outputID, count - this->_delay);
	}else{
	    buf[count] = nilObject;
	}
    }

    void
    Delay::request(int outputID, const ParameterSet &req)
    {
	NodeInput &nodeInput = this->inputs[this->_inputID];
        if (req.exist("LOOKAHEAD")) {
            int look = dereference_cast<int> (req.get("LOOKAHEAD")) - this->_delay;

            if (look > 0) {
                ParameterSet p;
                p.add("LOOKAHEAD", ObjectRef(Int::alloc(look)));
                nodeInput.node->request(nodeInput.outputID, p);
            }
        }
        if (req.exist("LOOKBACK")) {
            int look = dereference_cast<int> (req.get("LOOKBACK")) + this->_delay;

            if (look > 0) {
                ParameterSet p;
                p.add("LOOKBACK", ObjectRef(Int::alloc(look)));
                nodeInput.node->request(nodeInput.outputID, p);
            }
        }
    }
}

/* end of file */
